// components
(function ($, Drupal) {

  function init(editor) {

    // add new component type layout
    editor.DomComponents.addType('layout', {
      extend: 'row',
      model: {
        create(){
          editor.Panels.getPanel('spinner-loading').set('visible', true);
          var component = this;
          var data = {
            original: editor.BlockManager.get(component.attributes.type).attributes.additional.root,
            container: component.getContainer()
          };
          var parent = this.parent();
          var index = this.index();
          this.remove();

          Drupal.restconsumer.post('/pagedesigner/clone/', data).done(function (response) {
            var markup = response.filter(cmd => cmd.command == 'pd_markup')[0].data;
            var styles = response.filter(cmd => cmd.command == 'pd_styles')[0].data;

            parent.components().add(markup, { at: index });
            editor.Parser.parseCss(styles).forEach(function (rule) {
              if (!(Object.keys(rule.style).length === 0 && rule.style.constructor === Object)) {
                editor.CssComposer.setIdRule(rule.selectors[0], rule.style, { mediaText: rule.mediaText })
              }
            });
            parent.save();
            editor.Panels.getPanel('spinner-loading').set('visible', false);
          });
        }
      }
    });
  }

  Drupal.behaviors.pagedesigner_layouts_components = {
    attach: function (context, settings) {
      $(document).on('pagedesigner-init-components', function (e, editor) {
        init(editor);
      });
    }
  };

})(jQuery, Drupal);




// commands
(function ($, Drupal) {
  var initialized;

  function init(editor) {
    if (!initialized) {

      // command create layout
      editor.Commands.add('open-modal-create-layout', {
        run(editor, sender) {
          // open modal
          editor.Modal.setTitle('Create layout form component');
          var modal = $('#pdCreateLayout');
          if (modal.length == 0) {
            modal = $(`
            <div id="pdCreateLayout" title="Create layout form component" style="display: none;">
              <p>
              <i class="fas fa-copy" style="float:left; margin:12px 12px 20px 0;"></i>
              The component will be saved as a layout. Please give a name and a description.
              </p>
              <h4>Name*</h4>
              <input type="text" name="name" value="" />
              <h4>Description</h4>
              <textarea id="revisionMessage" name="description" style="height:150px;width: 100%;"></textarea>

            </div >
            `);
            $('body').append(modal);
          }
          modal.dialog({
            resizable: false,
            height: "auto",
            width: 400,
            modal: true,
            buttons: {
              "Save layout": (function (action) {
                return function () {
                  var name = modal.find("input[name=name]");
                  if (name.val() == '') {
                    name.css('border', '2px solid red');
                    return;
                  }
                  var description = modal.find("textarea[name=description]");
                  // var includeContent = modal.find("input[name=includecontent]");
                  Drupal.restconsumer.post('/pagedesigner/layout', {
                    original: editor.getSelected().get('entityId'),
                    name: name.val(),
                    description: description.val(),
                    // include_content: includeContent.get(0).checked,
                  }).done(function(response){

                    var pattern_id = 'layout' + response[0].id[0].value;
                    var layout_name = response[0].name[0].value;

                    editor.DomComponents.addType( pattern_id, {
                      extend: 'layout',
                      model: {
                        defaults: {
                          name: layout_name,
                          traits: [],
                          stylable: false,
                          styles: [],
                          toggable_classes: [],
                          styling_options: []
                        }
                      }
                    });

                    editor.BlockManager.add(pattern_id, {
                      additional: {
                        category: 'Layouts',
                        icon: 'fas fa-cube',
                        layout: response[0].id[0].value,
                        markup: '<div></div>',
                        pagedesigner: 1,
                        root: editor.getSelected().get('entityId'),
                        type: 'layout',
                      },
                      label: layout_name,
                      pattern: {
                        type: pattern_id,
                        template: '<div></div>'
                      },
                      content: '<div data-gjs-type="' + pattern_id + '"></div>',
                      preview: {},
                      category: 'Layouts',
                      attributes: {
                        class: 'far fa-sticky-note'
                      }
                    });
                  });
                  $(this).dialog("close");
                }
              })($(this)),
              Cancel: function () {
                $(this).dialog("close");
              }
            }
          });
        }
      });

      // command create layout
      editor.Commands.add('save-layout', {
        run(editor, sender) {

        }
      });

    }
  }

  Drupal.behaviors.pagedesigner_layouts_commands = {
    attach: function (context, settings) {
      $(document).on('pagedesigner-init-commands', function (e, editor) {
        init(editor);
      });
    }
  };

})(jQuery, Drupal);

// before init
(function ($, Drupal) {

  function init(editor) {

    // lock components for editing etc.
    editor.DomComponents.getTypes().forEach(function (componentType) {
      if (componentType.id == 'row' || componentType.id == 'component') {
        editor.DomComponents.addType(componentType.id, {
          model: componentType.model.extend(
            {
              initToolbar(...args) {
                componentType.model.__super__.initToolbar.apply(this, args);
                var tb = this.get('toolbar');
                if (tb) {
                  tb.push({
                    attributes: {
                      class: `fas fa-cube`,
                      title: Drupal.t('Store as layout component'),
                    },
                    command: 'open-modal-create-layout'
                  });
                  this.set('toolbar', tb);
                }
              },
            }
          ),
          view: componentType.view
        });
      }
    });
  }

  Drupal.behaviors.pagedesigner_layouts_before_init = {
    attach: function (context, settings) {
      $(document).on('pagedesigner-init-components', function (e, editor) {
        init(editor);
      });
    }
  };

})(jQuery, Drupal);




// after init
(function ($, Drupal) {

  function addDeleteButton(layout) {
    var block_elem = $('.gjs-block-category').find('.gjs-block-label:contains("' + layout.get('label') + '")').parent();
    if( block_elem.find('.gjs-am-close').length == 0 ){
      block_elem.append('<div class="gjs-am-close" data-toggle="layout-remove" data-layout="' + layout.get('id').replace('layout', '') + '">⨯</div>');
    }
  }

  function deleteLayout(layout_id){
    if (confirm("Do you really want to delete this layout?")) {
      editor.BlockManager.remove('layout' + layout_id)
      Drupal.restconsumer.delete('/pagedesigner/layout/' + layout_id);
    }
  }

  Drupal.behaviors.pagedesigner_layouts_after_init = {
    attach: function (context, settings) {
      // Add delete button to layout blocks
      $(document).on('pagedesigner-init-events', function (e, editor, options) {
        editor.on('run:open-blocks', () => {
          var layouts = editor.BlockManager.getAll().filter(layout => layout.get('category').id == 'Layouts');
          layouts.forEach(function(layout){
            addDeleteButton(layout);
          })
        });
      });

      $(document).on('click', '[data-toggle="layout-remove"]', function (e) {
        deleteLayout($(this).data('layout'))
      });
    }
  };

})(jQuery, Drupal);
