<?php
namespace Drupal\pagedesigner_layout\Plugin\pagedesigner\Handler;

use Drupal\pagedesigner\Definition\PatternDefinition;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\Plugin\pagedesigner\Handler\Standard;

/**
 * @PagedesignerHandler(
 *   id = "layout",
 *   name = @Translation("Layout handler"),
 *   types = {
 *      "layout"
 *   },
 * )
 */
class Layout extends Standard
{
    protected $definition = [
        "id" => 'layout',
        "pagedesigner" => 1,
        "icon" => "far fas fa-cube",
        "type" => "layout",
        "category" => 'Layouts',
        "label" => '',
        "description" => '',
    ];

    public function collectAttachments(&$attachments)
    {
        $attachments['library'][] = 'pagedesigner_layout/pagedesigner';
    }

    /**
     * {@inheritDoc}
     */
    public function collectPatterns(&$patterns)
    {
        $nids = \Drupal::entityQuery('pagedesigner_element')->condition('type', 'layout')->execute();
        $definitions = \Drupal::service('plugin.manager.ui_patterns')->getDefinitions();
        foreach ($nids as $nid) {
            $layout = Element::load($nid);
            if ($layout != null) {
                $root = $layout->children->entity;
                if ($root != null && !empty($definitions[$root->field_pattern->value])) {
                    if ($definitions[$root->field_pattern->value]->getProvider() === \Drupal::service('theme.manager')->getActiveTheme()->getName()) {
                        $definition = $this->definition;
                        $definition['id'] .= $nid;
                        $definition['layout'] = $nid;
                        $definition['markup'] = '<div></div>';
                        $definition['root'] = $root->id();
                        $definition['label'] = $layout->name->value;
                        $patterns[$definition['id']] = new PatternDefinition($definition);
                    }
                }
            }
        }
    }

    public function render(Element $entity)
    {
        return [];
    }

    /**
     * {@inheritdoc}
     */
    public function renderForEdit(Element $entity)
    {
        return [];
    }

}
