(function ($, Drupal) {
  Drupal.behaviors.pagedesigner_init_events = {
    attach: function (context, settings) {
      $(document).on('pagedesigner-init-events', function (e, editor, options) {

        editor.on('load', () => {

          // hacks - grapes bugs??
          editor.Panels.getPanel('component-controls').set('visible', true);
          editor.Panels.getPanel('component-controls').set('visible', false);

          editor.Panels.getPanel('views-container').set('visible', true);
          editor.Panels.getPanel('views-container').set('visible', false);


          editor.Panels.getPanel('spinner-loading').set('visible', true);
          editor.Panels.getPanel('spinner-loading').set('visible', false);

          const openStyles = editor.Panels.getButton('sidebar', 'sidebar-open-styles');
          openStyles && openStyles.set('active', true);

          const closeStyles = editor.Panels.getButton('sidebar', 'sidebar-open-styles');
          closeStyles && closeStyles.set('active', false);


          // prevent margin and padding from being grouped into shorthand definition
          editor.StyleManager.removeProperty('dimension', 'margin');
          editor.StyleManager.addProperty('dimension',{
            name: 'Margin top',
            property: 'margin-top',
            type: 'integer',
            units: ['px','%','em'],
            unit: 'px',
          });

          editor.StyleManager.addProperty('dimension',{
            name: 'Margin right',
            property: 'margin-right',
            type: 'integer',
            units: ['px','%','em'],
            unit: 'px',
          });

          editor.StyleManager.addProperty('dimension',{
            name: 'Margin bottom',
            property: 'margin-bottom',
            type: 'integer',
            units: ['px','%','em'],
            unit: 'px',
          });

          editor.StyleManager.addProperty('dimension',{
            name: 'Margin left',
            property: 'margin-left',
            type: 'integer',
            units: ['px','%','em'],
            unit: 'px',
          });


          editor.StyleManager.removeProperty('dimension', 'padding');
          editor.StyleManager.addProperty('dimension',{
            name: 'Padding top',
            property: 'padding-top',
            type: 'integer',
            units: ['px','%','em'],
            unit: 'px',
          });

          editor.StyleManager.addProperty('dimension',{
            name: 'Padding right',
            property: 'padding-right',
            type: 'integer',
            units: ['px','%','em'],
            unit: 'px',
          });

          editor.StyleManager.addProperty('dimension',{
            name: 'Padding bottom',
            property: 'padding-bottom',
            type: 'integer',
            units: ['px','%','em'],
            unit: 'px',
          });

          editor.StyleManager.addProperty('dimension',{
            name: 'Padding left',
            property: 'padding-left',
            type: 'integer',
            units: ['px','%','em'],
            unit: 'px',
          });





          // Object.assign({}


          var display_property = Object.assign({}, editor.StyleManager.getProperty('general', 'display').attributes);
          display_property.type = 'radio';
          display_property.full = 1;
          editor.StyleManager.removeProperty('general', 'display');
          editor.StyleManager.addProperty('general',display_property);

          editor.StyleManager.removeProperty('general', 'position');
          editor.StyleManager.addProperty('general',{
            name: 'Position',
            property: 'position',
            type: 'radio',
            defaults: 'static',
            fixedValues: [
              'initial',
              'inherit'
            ],
            list: [
              {value: "static"},
              {value: "relative"},
              {value: "absolute"},
              {value: "fixed"},
              {value: "sticky"},
            ]
          });


          var top_property = Object.assign({}, editor.StyleManager.getProperty('general', 'top').attributes);
          editor.StyleManager.removeProperty('general', 'top');
          editor.StyleManager.addProperty('general',top_property);

          var bottom_property = Object.assign({}, editor.StyleManager.getProperty('general', 'bottom').attributes);
          editor.StyleManager.removeProperty('general', 'bottom');
          editor.StyleManager.addProperty('general',bottom_property);

          var left_property = Object.assign({}, editor.StyleManager.getProperty('general', 'left').attributes);
          editor.StyleManager.removeProperty('general', 'left');
          editor.StyleManager.addProperty('general',left_property);

          var right_property = Object.assign({}, editor.StyleManager.getProperty('general', 'right').attributes);
          editor.StyleManager.removeProperty('general', 'right');
          editor.StyleManager.addProperty('general',right_property);

          editor.StyleManager.addProperty('general',{
            name: 'Z-Index',
            property: 'z-index',
            type: 'integer',
            defaults: '',
            min: -1
          });

          const panels = editor.Panels.getPanel('spinner-loading');
          var spinner_content = document.createElement('div');
          spinner_content.appendChild(jQuery('<i class="fas fa-spinner fa-spin"></i>').get(0));
          panels.set('appendContent', spinner_content).trigger('change:appendContent');


          // load styles and generate rules
          var pd_styles = jQuery('#pd_styles').html();
          if (pd_styles) {

            editor.Parser.parseCss(pd_styles).forEach(function (rule) {
              if (!(Object.keys(rule.style).length === 0 && rule.style.constructor === Object)) {
                editor.CssComposer.setIdRule(rule.selectors[0], rule.style, { mediaText: rule.mediaText })
              }
            });
            jQuery('#pd_styles').remove();
          }

          // load stylesheets into canvas iframe and attach body classes
          document.querySelectorAll("link[rel=stylesheet]").forEach(function (css_file) {
            editor.Canvas.getDocument().head.appendChild(css_file.cloneNode(true))
          });
          Array.from(document.querySelectorAll("style")).forEach(function (style) {
            editor.Canvas.getDocument().head.appendChild(style.cloneNode(true));
          });
          editor.Canvas.getDocument().body.className += document.body.className;

          document.querySelector("iframe.gjs-frame").contentWindow.drupalSettings = window.drupalSettings;

          // load scripts into canvas
          document.querySelectorAll('script[src]:not([src*="grapes"])').forEach(function (script) {
            if (
              script.src.indexOf('/drupalSettingsLoader') === -1 &&
              script.src.indexOf('/pagedesigner.js') === -1 &&
              script.src.indexOf('/pagetree/') === -1 &&
              script.src.indexOf('/dialog.js') === -1
            ) {
              var newScript = document.createElement('script');
              newScript.setAttribute('src', script.src);
              editor.Canvas.getDocument().body.appendChild(newScript)
            }
          });

          // prevent links from being opened inside editor canvas
          editor.Canvas.getDocument().querySelectorAll("a").forEach(function (link) {
            link.target = '_parent';
          });

          editor.LayerManager.setRoot('[data-grapes-block="content"]');

          jQuery(editor.Canvas.getDocument().querySelectorAll("[data-gjs-type='container']")).on('click', 'a', function (e) {
            e.preventDefault();
          });

          // lock all classes from patterns from being editable
          var t0 = performance.now();
          var classes = [];
          var patterns = '';
          editor.BlockManager.getAll().models.forEach(function (block) {
            patterns += block.get('pattern').template;
          })

          var matches = patterns.match(/ class=\"([A-Za-z0-9\- _]*)\"/g);
          if (matches) {
            matches.forEach(function (match) {
              classes = classes.concat(match.substr(8, match.length - 9).split(" "));
            });
          }

          classes = Array.from(new Set(classes));

          classes.forEach(function (className) {
            editor.SelectorManager.addClass(className);
            editor.SelectorManager.get('.' + className).set('private', true).set('protected', true);
          })
          var t1 = performance.now();


          editor.SelectorManager.getAll().each(selector => selector.set('active', false));


          jQuery(document).on('change', '[data-toggle-class]', function () {
            var className = jQuery(this).val();
            if (jQuery(this).data('responsive')) {
              className += '-' + editor.getConfig().deviceManager.devices.filter(device => device.name == editor.getDevice()).key;
            }
            editor.SelectorManager.addClass(className);
            editor.SelectorManager.get('.' + className).set('private', true).set('protected', true).set('active', false);
            if (editor.getSelected().getClasses().includes(className)) {
              editor.getSelected().removeClass(className);
            } else {
              editor.getSelected().addClass(className);
            }
          });


          $(document).on('change', '[data-select-class]', function () {
            component = editor.getSelected();
            var class_name = $(this).val();
            for( remove_class_name in component.get('styling_options')[$(this).data('styling-option')].options){
              component.removeClass(remove_class_name);
            }
            if( class_name ){
              component.addClass(class_name);
            }
          });

        });

        editor.on('component:selected', (component, sender) => {

          component.setStylableProperties();

          if (component.load && Drupal.restconsumer) {
            component.load();
          }

          // Begin Class toggle
          if (jQuery('.gjs-clm-class-toggle').length == 0) {
            jQuery('<div class="gjs-clm-class-toggle gjs-one-bg gjs-two-color"><div data-class-toggle-container></div></div>').insertBefore('.gjs-clm-tags')
          }

          jQuery('[data-class-toggle-container]').html('');

          var classesGlobal = '';
          var classesResponsive = '';
          for (var className in editor.getSelected().get('toggable_classes')) {
            var classTMP = editor.getSelected().get('toggable_classes')[className];
            if (classTMP.responsive) {
              classesResponsive += '<div class="responsive-class"><p>' + classTMP.label + ' <a class="info" title="' + classTMP.description + '"><i class="fas fa-info-circle"></i></a></p>';
              editor.getConfig().deviceManager.devices.forEach(function (device) {
                var checked = '';
                if (editor.getSelected().getClasses().includes(className + '-' + device.key)) {
                  checked = 'checked="checked" '
                }
                var deviceactive = '';
                if (editor.getDevice() == device.name) {
                  deviceactive = ' active';
                }

                if( device.name == 'Desktop'){
                  var icon = device.name.toLowerCase();
                }else if( device.name == 'Tablet'){
                  var icon = device.name.toLowerCase() + '-alt';
                }else{
                  var icon = 'mobile-alt';
                }

                classesResponsive += '<label data-device="' + device.name + '" class="inline-label' + deviceactive + '"><input type="checkbox" data-toggle-class name="toggable_classes[]" ' + checked + ' value="' + className + '-' + device.key + '"/><span title="' + device.name + '" class="gjs-pn-btn fas fa-' + icon + '"></span></label>';
              });
              classesResponsive += '</div>';
            } else {
              var strTMP = 'value="' + className + '" ';
              if (editor.getSelected().getClasses().includes(className)) {
                strTMP += 'checked="checked" '
              }
              classesGlobal += '<label title="' + classTMP.description + '"><input type="checkbox" data-toggle-class name="toggable_classes[]" ' + strTMP + '/>' + classTMP.label + ' <a class="info" title="' + classTMP.description + '"><i class="fas fa-info-circle"></i></a></label>';
            }
          }

          if (classesGlobal) {
            jQuery('[data-class-toggle-container]').append('<div class="options-holder"><p class="sidebar-subtitle">Global options</p>' + classesGlobal + '</div>');
          }

          if (classesResponsive) {
            jQuery('[data-class-toggle-container]').append('<div class="options-holder" data-responsive-options><p class="sidebar-subtitle">Responsive options</p>' + classesResponsive + '</div>');
          }

          // End Class toggle

          // Begin easy styling options

          if (jQuery('.gjs-clm-easy-styling').length == 0) {
            jQuery('<div class="gjs-clm-easy-styling gjs-one-bg gjs-two-color" data-easy-styling-container></div>').insertBefore('.gjs-clm-tags')
          }

          jQuery('[data-easy-styling-container]').html('');

          var styling_options_html = '';
          var styling_options = component.get('styling_options');
          var selected_classes = component.getClasses();
          for (var styling_option in styling_options ) {

            styling_options_html += '<div class="options-holder" ><p >' + styling_options[styling_option].label + '</p><select data-select-class data-styling-option="' + styling_option + '"><option value="">None</option>';

            for (var class_name in styling_options[styling_option].options ){
              if( selected_classes.includes(class_name) ){
                styling_options_html += '<option value="' + class_name + '" selected>' + styling_options[styling_option].options[class_name] + '</option>';
              }else{
                styling_options_html += '<option value="' + class_name + '">' + styling_options[styling_option].options[class_name] + '</option>';
              }
            }

            styling_options_html += '</select></div>';
          }

          if( styling_options_html ){
            jQuery('[data-easy-styling-container]').append( '<p class="sidebar-subtitle">' + Drupal.t('Easy styling options') + '</p><div >' + styling_options_html + '</div>' );
          }









          // end easy styling options

        });

        editor.on('component:deselected', (component, sender) => {
          if (component.get('changed')) {
            if (confirm("Do you want to save your changes?")) {
              component.save();
            } else {
              component.restore();
            }
          }
        });

        editor.on('component:update:classes', component => {
          component.set('changed', true);
          component.getClasses().forEach(function(className){
            editor.SelectorManager.get('.' + className).set('active', false);
          });
        });

        editor.on('run:open-blocks', component => {
          if( $('#filter-blocks').length < 1 ){
            $('.gjs-block-categories').prepend('<label class="filter-blocks-holder"><input id="filter-blocks" placeholder="Filter components" /></label>');

            $('#filter-blocks').keyup(function(){
              var filter_value = $(this).val().toLowerCase();

              if( filter_value ){
                $('.gjs-block-categories').find('.gjs-blocks-c').children('[title]').hide();
                $('.gjs-block-categories').find('.gjs-blocks-c').children('[title]').each(function(){
                  if( $(this).attr('title').toLowerCase().indexOf(filter_value) !== -1 ){
                    $(this).show()
                  }
                });
              }else{
                $('.gjs-block-categories').find('.gjs-blocks-c').children('[title]').show()
              }
            });
          }
        });


        editor.on('run:open-tm', component => {
          editor.Panels.getPanel('views-container').set('visible', true);
          editor.Panels.getPanel('component-controls').set('visible', true);

          var traitsHolder = $('.gjs-blocks-cs').parent();
          if( traitsHolder.find('.sidebar-title').length == 0 ){
            traitsHolder.prepend('<p class="sidebar-title">' + Drupal.t('Edit component content') + '</p>')
          }

        });

        editor.on('stop:open-tm', component => {
          editor.Panels.getPanel('views-container').set('visible', false);
          editor.Panels.getPanel('component-controls').set('visible', false);
          var traitsHolder = $('.gjs-trt-traits').parent();
          if( traitsHolder.find('.sidebar-title').length == 0 ){
            $('.gjs-traits-label').remove();
            traitsHolder.prepend('<p class="sidebar-title">' + editor.TraitManager.getConfig().labelContainer + '</p>')
          }
        });

        editor.on('run:open-sm', component => {
          editor.Panels.getPanel('views-container').set('visible', true);
          editor.Panels.getPanel('component-controls').set('visible', true);

          var stylesHolder = $('.gjs-clm-tags').parent();
          if( stylesHolder.find('.sidebar-title').length == 0 ){
            $('.gjs-traits-label').remove();
            stylesHolder.prepend('<p class="sidebar-title">' + Drupal.t('Edit component styling') + '</p>');
            $('#gjs-clm-label').addClass('sidebar-subtitle').text( Drupal.t('Custom CSS classes') );

            if( $('.gjs-sm-sectors').find('.sidebar-subtitle').length == 0 ){
              $('.gjs-sm-sectors').prepend('<div class="sectors-title-holder"><p class="sidebar-subtitle">' + Drupal.t('Advanced styling options') + '</p><p><strong>Use with caution.</strong><br/>All previously selected styling options can be overridden by directly customizing the following CSS properties.</p></div>' );
            }

          }
        });

        editor.on('stop:open-sm', component => {
          editor.Panels.getPanel('views-container').set('visible', false);
          editor.Panels.getPanel('component-controls').set('visible', false);
        });

        editor.on('run:open-blocks', component => {
          editor.Panels.getPanel('views-container').set('visible', true);
          var blocksHolder = $('.gjs-blocks-cs').parent();
          if( blocksHolder.find('.sidebar-title').length == 0 ){
            blocksHolder.prepend('<p class="sidebar-title">' + editor.BlockManager.getConfig().labelContainer + '</p>')
          }
        });

        editor.on('stop:open-blocks', component => {
          editor.Panels.getPanel('views-container').set('visible', false);
        });

        // prevent layer manager from selecting components when toggling children
        editor.on('run:open-layers', component => {

          var layerssHolder = $('.gjs-layer:first').parent();
          if( layerssHolder.find('.sidebar-title').length == 0 ){
            layerssHolder.prepend('<p class="sidebar-title">' + Drupal.t('Layers') + '</p>')
          }

          jQuery('.gjs-layer [data-toggle-select]').removeAttr('data-toggle-select');
          jQuery('.gjs-layer [data-name]').attr('data-toggle-select', 1);
          jQuery('.fa.fa-arrows').addClass('fa-arrows-alt');
          editor.Panels.getPanel('views-container').set('visible', true);
        });

        editor.on('stop:open-layers', component => {
          editor.Panels.getPanel('views-container').set('visible', false);
        });

        editor.on('component:add', component => {
          if (component.create) {
            component.create();
          }
        });

        editor.on('component:update:components', (component, previousAttributes) => {
          if (component.get('entityId') && component.attributes.droppable && Object.keys(previousAttributes.components()._byId).length > 0 && component.save) {
            component.save();
          }
        });



      });
    }
  };
})(jQuery, Drupal);
