<?php
namespace Drupal\pagedesigner\Plugin\DataType;

use Drupal\Core\TypedData\TypedData;
use Drupal\pagedesigner\Entity\Element;

/**
 * A computed property for pagedesigner content data.
 *
 * @DataType(
 *   id = "pagedesigner_item_data",
 *   label = @Translation("Pagedesigner content data"),
 *   description = @Translation("Generated content of the pagedesigner."),
 * )
 */
class PagedesignerData extends TypedData
{

    /**
     * Cached processed text.
     *
     * @var string|null
     */
    protected $value = null;

    /**
     * Cached processed text.
     *
     * @var string|null
     */
    protected $processed = null;

    /**
     * {@inheritdoc}
     */
    public function getValue()
    {
        $language = $this->getParent()->getEntity()->language()->getId();
        if ($this->processed == null) {
            $this->processed = $this->getContent($this->getParent()->target_id, $language);
        }
        return $this->processed;
    }

    public function getContent($containerId, $langcode)
    {
        if (\is_numeric($containerId)) {
            $container = Element::load($containerId);
            if ($container != null && $container->hasTranslation($langcode)) {
                $container = $container->getTranslation($langcode);
                $handler = \Drupal::service('plugin.manager.pagedesigner_handler')->getInstance(['type' => $container->bundle()])[0];
                $list = [];
                $handler->getContent($container, $list);
                $this->processed = $list;
            }
        }
        return $this->processed;
    }

    /**
     * {@inheritdoc}
     */
    public function setValue($value, $notify = true)
    {
        $this->processed = $value;

        // Notify the parent of any changes.
        if ($notify && isset($this->parent)) {
            $this->parent->onChange($this->name);
        }
    }

    /**
     * {@inheritdoc}
     */
    public function getString()
    {
        return implode(' ', $this->getValue());
    }

}
