<?php

namespace Drupal\pagedesigner\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;

/**
 * Plugin implementation of the Pagedesigner content formatter.
 *
 * @FieldFormatter(
 *   id = "pagedesigner_formatter",
 *   module = "pagedesigner",
 *   label = @Translation("Pagedesigner content"),
 *   field_types = {
 *     "pagedesigner_item"
 *   }
 * )
 */
class PagedesignerFormatter extends FormatterBase
{

    /**
     * {@inheritdoc}
     */
    public function settingsSummary()
    {
        $summary = [];
        $summary[] = $this->t('Displays the pagedesigner content.');
        return $summary;
    }

    /**
     * {@inheritdoc}
     */
    public function viewElements(FieldItemListInterface $items, $langcode)
    {
        $elements = [];
        $node = $items->getEntity();
        if ($node != null) {
            foreach ($items as $delta => $item) {
                $container = $item->entity;
                if ($container != null && $container->hasTranslation($langcode)) {
                    $container = $container->getTranslation($langcode);
                    $renderer = \Drupal::service('pagedesigner.service.renderer');
                    if (\Drupal::currentUser()->hasPermission('edit pagedesigner element entities') && isset($_GET['pd']) && $_GET['pd'] == 1) {
                        $renderer->renderForEdit($container, $node);
                    } elseif (\Drupal::currentUser()->hasPermission('view unpublished pagedesigner element entities')) {
                        $renderer->render($container, $node);
                    } else {
                        $renderer->renderForPublic($container, $node);
                    }
                    $markup = $renderer->getMarkup();
                    $elements[] = ['#markup' => $markup];
                }
            }
        }
        return $elements;
    }

}
