<?php
namespace Drupal\pagedesigner\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\pagedesigner\Entity\Element;

/**
 * Plugin implementation of the Pagedesigner content field type.
 *
 * @FieldType(
 *   id = "pagedesigner_item",
 *   label = @Translation("Pagedesigner content"),
 *   module = "pagedesigner",
 *   description = @Translation("Displays the pagedesigner content for this node."),
 *   default_widget = "pagedesigner_entity_reference_autocomplete",
 *   default_formatter = "pagedesigner_formatter",
 *   translatable = true,
 *   list_class = "\Drupal\Core\Field\EntityReferenceFieldItemList",
 * )
 */
class PagedesignerItem extends EntityReferenceItem
{

    public function getValue()
    {
        $values = parent::getValue();
        $language = $this->getParent()->getEntity()->language()->getId();
        $values['content'] = $this->properties['content']->getContent($values['target_id'], $language);
        return $values;
    }

    /**
     * {@inheritdoc}
     */
    public function setValue($values, $notify = true)
    {
        $language = $this->getParent()->getEntity()->language()->getId();
        if (isset($values['target_id'])) {
            $values['content'] = $this->getContent($values['target_id'], $language);
        }
        parent::setValue($values, $notify);
    }

    /**
     * Undocumented function
     *
     * @param [type] $containerId
     * @param [type] $langcode
     * @return void
     */
    public function getContent($containerId, $langcode)
    {
        $content = '';
        if (\is_numeric($containerId)) {
            $container = Element::load($containerId);
            if ($container != null && $container->hasTranslation($langcode)) {
                $container = $container->getTranslation($langcode);
                $handler = \Drupal::service('plugin.manager.pagedesigner_handler')->getInstance(['type' => $container->bundle()])[0];
                $list = [];
                $handler->getContent($container, $list);
                $content = implode(' ', $list);
            }
        }
        return $content;
    }

    /**
     * {@inheritdoc}
     */
    public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition)
    {
        $properties = parent::propertyDefinitions($field_definition);
        $properties['content'] = DataDefinition::create('pagedesigner_item_data')
            ->setLabel(new TranslatableMarkup('Pagedesigner content'))
            ->setComputed(true)
            ->setClass('\Drupal\pagedesigner\Plugin\DataType\PagedesignerData')
            ->setInternal(false);
        return $properties;
    }

    /**
     * {@inheritdoc}
     */
    public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data)
    {
        $element = parent::storageSettingsForm($form, $form_state, $has_data);
        $element['target_type']['#options'] = ['pagedesigner_element' => $element['target_type']['#options']['Content']['pagedesigner_element']];
        return $element;
    }

    /**
     * {@inheritdoc}
     */
    public function fieldSettingsForm(array $form, FormStateInterface $form_state)
    {
        $form = parent::fieldSettingsForm($form, $form_state);
        $form['handler']['handler']['#options'] = ['default:pagedesigner_element' => $form['handler']['handler']['#options']['default:pagedesigner_element']];
        $form['handler']['handler_settings']['auto_create']['#type'] = 'hidden';
        $form['handler']['handler_settings']['target_bundles']['#default_value'] = ['container'];
        $form['handler']['handler_settings']['target_bundles']['#options'] = ['container' => $form['handler']['handler_settings']['target_bundles']['#options']['container']];
        return $form;

    }
}
