<?php

namespace Drupal\pagedesigner\Plugin\pagedesigner;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\pagedesigner\Entity\Element;
use Drupal\ui_patterns\Definition\PatternDefinitionField;

/**
 * Povides the interface for defining handlers of pagedesigner elements.
 */
interface HandlerPluginInterface extends PluginInspectionInterface, ContainerFactoryPluginInterface
{
    /**
     * Collect attachments needed for editing inside the pagedesigner.
     *
     * @param array $attachments The list of attachments as an array.
     * @return void
     */
    public function collectAttachments(&$attachments);

    /**
     * Allows to alter the list of patterns generated by the ui_pattern module.
     *
     * @param \Drupal\ui_patterns\Definition\PatternDefinition[] $patterns
     * @return void
     */
    public function collectPatterns(&$patterns);

    /**
     * Processes a field.
     *
     * @param \Drupal\ui_patterns\Definition\PatternDefinitionField $field
     * @param array $fieldArray
     * @return void
     */
    public function prepare(PatternDefinitionField &$field, &$fieldArray);

    /**
     * Get a textual representation of the entity.
     *
     * @param \Drupal\pagedesigner\Entity\Element $entity The entity to get the value from.
     * @return string
     */
    public function get(Element $entity);

    /**
     * Update the entity.
     *
     * @param \Drupal\pagedesigner\Entity\Element $entity The entity to update.
     * @param string $data The data to update the entity with.
     * @return void
     */
    public function patch(Element $entity, $data);

    /**
     * Generate a new entity.
     *
     * @param array $definition The definition of the new entity.
     * @param string $data The data to create the entity with.
     * @return void
     */
    public function generate($definition, $data);

    /**
     * Copy an entity and return the cloned entity.
     *
     * @param \Drupal\pagedesigner\Entity\Element $entity The entity to copy.
     * @param \Drupal\Core\Entity\ContentEntityBase $container The container for the cloned entity.
     * @return \Drupal\pagedesigner\Entity\Element
     */
    public function copy(Element $entity, Element $container = null);

    /**
     * Delete an entity.
     *
     * @param \Drupal\pagedesigner\Entity\Element $entity The entity to delete.
     * @return void
     */
    public function delete(Element $entity);

    /**
     * Render an entity for authenticated users.
     *
     * @param \Drupal\pagedesigner\Entity\Element $entity
     * @return array The render array.
     */
    public function render(Element $entity);

    /**
     * Render an entity for anonymous users.
     *
     * @param \Drupal\pagedesigner\Entity\Element $entity
     * @return array The render array.
     */
    public function renderForPublic(Element $entity);

    /**
     * Render an entity for editing.
     *
     * @param \Drupal\pagedesigner\Entity\Element $entity
     * @return array The render array.
     */
    public function renderForEdit(Element $entity);
}
