<?php

namespace Drupal\pagedesigner\Service;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\PagedesignerService;

class StateChanger extends PagedesignerService
{
    /**
     * Publish an entity
     *
     * @param ContentEntityBase $entity
     * @return void
     */
    public function publish(ContentEntityBase $entity = null)
    {
        if ($entity == null) {
            return $this;
        }
        $fields = $this->getPagedesignerFields($entity);
        foreach ($fields as $field) {
            $container = $this->getContainer($entity, $field->getName());
            $this->_output .= $this->getHandler($container)->publish($container);
        }
        return $this;
    }

    /**
     * Unpublish an entity
     *
     * @param ContentEntityBase $entity
     * @return void
     */
    public function unpublish(ContentEntityBase $entity = null)
    {
        if ($entity == null) {
            return $this;
        }
        $fields = $this->getPagedesignerFields($entity);
        foreach ($fields as $field) {
            $container = $this->getContainer($entity, $field->getName());
            $this->_output = $this->getHandler($container)->unpublish($container);
        }
        return $this;
    }

    /**
     * {@inheritDoc}
     */
    public function copy(Element $entity, Element $container = null)
    {
        if ($entity == null) {
            return $this;
        }
        $clone = $this->getHandler($entity)->copy($entity, $container);
        $this->_output = $clone;
        return $this;
    }

    /**
     * {@inheritDoc}
     */
    public function generate(ContentEntityBase $entity = null)
    {
        if ($entity == null) {
            return $this;
        }
        $this->_output = $this->getContainer($entity)->id();
        return $this;
    }

    /**
     * {@inheritDoc}
     */
    public function delete(ContentEntityBase $entity = null)
    {
        if ($entity == null) {
            return $this;
        }
        $fields = $this->getPagedesignerFields($entity);
        foreach ($fields as $field) {
            $container = $this->getContainer($entity, $field->getName());
            $this->_output = $this->getHandler($container)->delete($container);
        }
        return $this;
    }
}
